/*
** Copyright 2017-2019 Double Precision, Inc.
** See COPYING for distribution information.
*/
#include "libcxxw_config.h"
#include "x/w/uielements.H"
#include "x/w/uigenerators.H"
#include "messages.H"
#include "gridlayoutmanager.H"
#include "x/w/booklayoutmanager.H"
#include "x/w/impl/container.H"
#include "x/w/radio_group.H"
#include "screen.H"
#include "defaulttheme.H"

LIBCXXW_NAMESPACE_START

typedef uielements::new_radio_groups_t new_radio_groups_t;

new_radio_groups_t::new_radio_groups_t()=default;

new_radio_groups_t::~new_radio_groups_t()=default;

new_radio_groups_t::new_radio_groups_t(const new_radio_groups_t &)=default;

new_radio_groups_t::new_radio_groups_t(new_radio_groups_t &&)=default;

new_radio_groups_t &new_radio_groups_t::operator=(const new_radio_groups_t &)
=default;

new_radio_groups_t &new_radio_groups_t::operator=(new_radio_groups_t &&)
=default;

uielements::~uielements()=default;

element uielements::get_element(const std::string_view &name) const
{
	// TODO: C++20;

	auto iter=new_elements.find(std::string{name.begin(), name.end()});

	if (iter == new_elements.end())
		throw EXCEPTION(gettextmsg(_("Element %1% was not found"),
					   name));

	return iter->second;
}

radio_group uielements::get_radio_group(const std::string_view &name) const
{
	// TODO: C++20;

	auto iter=new_radio_groups.find(std::string{name.begin(), name.end()});

	if (iter == new_radio_groups.end())
		throw EXCEPTION(gettextmsg
				(_("Radio button group %1% was not found"),
				 name));

	return iter->second;
}

/////////////////////////////////////////////////////////////////////////////

void gridlayoutmanagerObj::generate(const std::string_view &name,
				    const const_uigenerators &generators,
				    uielements &elements)
{
	auto iter=generators->gridlayoutmanager_generators.find({name.begin(),
								 name.end()});

	if (iter == generators->gridlayoutmanager_generators.end())
	{
		throw EXCEPTION(gettextmsg(_("Layout %1% not defined."),
					   name));
	}

	auto me=ref{this};

	for (const auto &g:*iter->second)
		g(me, elements);
}

void booklayoutmanagerObj::generate(const std::string_view &name,
				    const const_uigenerators &generators,
				    uielements &elements)
{
	auto iter=generators->booklayoutmanager_generators.find({name.begin(),
								 name.end()});

	if (iter == generators->booklayoutmanager_generators.end())
	{
		throw EXCEPTION(gettextmsg(_("Layout %1% not defined."),
					   name));
	}

	auto me=ref{this};

	for (const auto &g:*iter->second)
		g(me, elements);
}

LIBCXXW_NAMESPACE_END
